# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Folder.password'
        db.alter_column('rgallery_folder', 'password', self.gf('django.db.models.fields.CharField')(max_length=255, null=True))

    def backwards(self, orm):

        # Changing field 'Folder.password'
        db.alter_column('rgallery_folder', 'password', self.gf('django.db.models.fields.CharField')(default='', max_length=255))

    models = {
        'rgallery.folder': {
            'Meta': {'object_name': 'Folder'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'rgallery.photo': {
            'Meta': {'object_name': 'Photo'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'rgallery.video': {
            'Meta': {'object_name': 'Video'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'video': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'blank': 'True'})
        }
    }

    complete_apps = ['rgallery']


# -*- coding: utf-8 -*-

import os
import json
import zipfile
import urllib2
from StringIO import StringIO

from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
import models as mymodels

from braces.views import LoginRequiredMixin, SuperuserRequiredMixin
from taggit.models import Tag

from django.conf import settings as conf

from django.views.generic import ListView, CreateView

from engine import expire_view_cache
from management.commands.utils import *
from forms import PhotoForm


class TagMixin(object):
    def get_context_data(self, **kwargs):
        context = super(TagMixin, self).get_context_data(**kwargs)
        if self.request.user.is_superuser:
            context['tags'] = Tag.objects.all()
        else:
            tags = Tag.objects.filter(photo__status=True)
            context['tags'] = tags.distinct()
        return context


class Photos(TagMixin, ListView):

    template_name = "rgallery/photos.html"
    context_object_name = "photos"
    paginate_by = 150

    def get_queryset(self):
        if self.request.user.is_superuser:
            return mymodels.Photo.objects.all().order_by('-capture_date')
        else:
            try:
                return mymodels.Photo.objects.all().filter(
                    status=1,
                    folder__isnull=True,
                    tags__slug=conf.HOMETAG).order_by('-capture_date')
            except:
                return mymodels.Photo.objects.all().filter(
                    status=1,
                    folder__isnull=True).order_by('-capture_date')

    def get_context_data(self, **kwargs):
        context = super(Photos, self).get_context_data(**kwargs)
        folders = mymodels.Folder.objects.filter(status=1)
        form = PhotoForm

        context.update({
            'title': _(u'Photos'),
            'form': form,
            'folders': folders,
        })
        return context


class PhotosFolder(ListView):

    template_name = "rgallery/photos.html"
    context_object_name = "photos"

    def get_queryset(self):
        folder = mymodels.Folder.objects.get(slug=self.kwargs['folder'])
        return mymodels.Photo.objects.all().filter(
            status=1,
            folder=folder).order_by('-capture_date')

    def get_context_data(self, **kwargs):
        context = super(PhotosFolder, self).get_context_data(**kwargs)

        context.update({
            'title': _(u'Photos'),
        })
        return context


class PhotosTag(ListView):

    template_name = "rgallery/photos.html"
    model = mymodels.Photo
    context_object_name = "photos"
    paginate_by = 150

    def get_queryset(self):
        if self.request.user.is_superuser:
            return mymodels.Photo.objects.filter(
                tags__slug=self.kwargs.get(
                    'slug', None)).order_by('-capture_date')
        else:
            return mymodels.Photo.objects.filter(
                status=1,
                tags__slug=self.kwargs.get(
                    'slug', None)).order_by('-capture_date')

    def get_context_data(self, **kwargs):
        context = super(PhotosTag, self).get_context_data(**kwargs)

        context.update({
            'tag': self.kwargs.get('slug', None),
        })
        return context


class PhotoAddTag(LoginRequiredMixin, SuperuserRequiredMixin, ListView):

    model = mymodels.Photo

    def post(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'POST':
            pk = json.loads(self.request.POST.get('pk', None))
            tags = json.loads(self.request.POST.get('tags', None))
            response_data = {}
            if type(pk) is list:
                # List
                photos = mymodels.Photo.objects.filter(id__in=pk)
                for photo in photos:
                    photo.tags.clear()
                    if len(tags) > 0:
                        for tag in tags:
                            try:
                                t = Tag.objects.get(slug=tag)
                                photo.tags.add(t)
                            except:
                                photo.tags.add(tag)
                    photo.save()
                    key = 'html-%s' % photo.pk
                    response_data[key] = render_to_string(
                        'rgallery/_listags.html',
                        {'record': photo.tags.all()})
                response_data['multiple'] = 1
                response_data['pk'] = pk
            else:
                # Id
                photo = get_object_or_404(mymodels.Photo, pk=pk)
                photo.tags.clear()
                if len(tags) > 0:
                    for tag in tags:
                        try:
                            t = Tag.objects.get(slug=tag)
                            photo.tags.add(t)
                        except:
                            photo.tags.add(tag)
                photo.save()
                response_data['multiple'] = 0
                response_data['html'] = render_to_string(
                    'rgallery/_listags.html',
                    {'record': photo.tags.all()})
            expire_view_cache("app_gallery-gallery")
            return HttpResponse(json.dumps(response_data),
                                content_type="application/json")


class PhotoAdd(LoginRequiredMixin, SuperuserRequiredMixin, CreateView):

    model = mymodels.Photo
    form_class = PhotoForm
    success_url = '/'

    def form_valid(self, form):
        self.object = form.save(commit=False)

        backend = __import__('rgallery.management.commands.backend_form',
                             fromlist='*')

        # Set dirs (src, dest)
        srcdir, photodestdir, videodestdir = backend.set_dirs(
            form.files['file'])

        img_duped, img_total, vid_duped, vid_total, total = mediasync(
            form.files['file'],
            srcdir,
            photodestdir,
            videodestdir,
            conf.RGALLLERY_THUMBS,
            backend,
            False,
            0,
            0,
            0,
            0,
            0)

        expire_view_cache("app_gallery-gallery")
        return HttpResponseRedirect(self.get_success_url())


class PhotoDelete(LoginRequiredMixin, SuperuserRequiredMixin, ListView):

    model = mymodels.Photo
    template_name = "rgallery/photo_delete.html"

    def get(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'GET':
            pk = json.loads(self.request.GET.get('pk', None))
            response_data = {}
            if type(pk) is list:
                # List
                photos = mymodels.Photo.objects.filter(id__in=pk)
                for photo in photos:
                    photo.delete()
                response_data['pk'] = pk
                response_data['multiple'] = 1
            else:
                # Id
                photo = get_object_or_404(mymodels.Photo, pk=pk)
                photo.delete()
                response_data['pk'] = pk
                response_data['multiple'] = 0
            expire_view_cache("app_gallery-gallery")
            return HttpResponse(json.dumps(response_data),
                                content_type="application/json")


class PhotoDownload(LoginRequiredMixin, ListView):

    model = mymodels.Photo
    template_name = "rgallery/photo_delete.html"

    def render_to_response(self, context):
        pk = urllib2.unquote(self.request.COOKIES.get("selected", None))
        pk = filter(None, pk.split(','))
        # List
        s = StringIO()
        zf = zipfile.ZipFile(s, "w")

        photos = mymodels.Photo.objects.filter(id__in=pk)
        for photo in photos:
            fdir, fname = os.path.split(photo.image.path)
            zf.write(photo.image.path, fname)
        zf.close()
        resp = HttpResponse(s.getvalue(),
                            mimetype="application/x-zip-compressed")
        resp['Content-Disposition'] = 'attachment; filename=photos.zip'
        return resp


class PhotoChangeStatus(LoginRequiredMixin, SuperuserRequiredMixin, ListView):

    model = mymodels.Photo
    template_name = "rgallery/photo_delete.html"

    def get(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'GET':
            pk = json.loads(self.request.GET.get('pk', None))
            status = json.loads(self.request.GET.get('status', None))
            response_data = {}
            if type(pk) is list:
                # List
                photos = mymodels.Photo.objects.filter(id__in=pk)
                for photo in photos:
                    photo.status = status
                    photo.save()
                response_data['pk'] = pk
                response_data['status'] = status
                response_data['multiple'] = 1
            else:
                # Id
                photo = get_object_or_404(mymodels.Photo, pk=pk)
                if photo.status == 1:
                    photo.status = 0
                else:
                    photo.status = 1
                photo.save()
                response_data['pk'] = pk
                response_data['status'] = photo.status
                response_data['multiple'] = 0
            expire_view_cache("app_gallery-gallery")
            return HttpResponse(json.dumps(response_data),
                                content_type="application/json")


class PhotoGetVideo(LoginRequiredMixin, ListView):

    model = mymodels.Photo

    def post(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'POST':
            pk = json.loads(self.request.POST.get('pk', None))
            photo = get_object_or_404(mymodels.Photo, pk=pk)
            rendered = render_to_string('rgallery/_video.html',
                                        {'video': photo.video})
        return HttpResponse(rendered)


class Videos(ListView):

    template_name = "rgallery/videos.html"
    context_object_name = "videos"
    paginate_by = 150

    def get_queryset(self):
        vid = mymodels.Video.objects.all().filter(
            status=1, folder__isnull=True).order_by('-capture_date')
        print vid
        return vid

    def get_context_data(self, **kwargs):
        context = super(Videos, self).get_context_data(**kwargs)
        context.update({
            'title': _(u'Videos'),
        })
        return context



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *
from rgallery.views import (Photos,
                            Videos,
                            PhotosFolder,
                            PhotoDelete,
                            PhotoAdd,
                            PhotosTag,
                            PhotoAddTag,
                            PhotoGetVideo,
                            PhotoChangeStatus,
                            PhotoDownload)

urlpatterns = patterns('',
    url(r'^$',
        Photos.as_view(),
        name='app_gallery-gallery'),
    url(r'^page/(?P<page>\d+)/$',
        Photos.as_view(),
        name='app_gallery-gallery-page'),
    url(r'^photo/del/$',
        PhotoDelete.as_view(),
        name='app_gallery-photo-del'),
    url(r'^photo/add/$',
        PhotoAdd.as_view(),
        name='app_gallery-photo-add'),
    url(r'^photo/changestatus/$',
        PhotoChangeStatus.as_view(),
        name='app_gallery-photo-changestatus'),
    url(r'^photo/download/$',
        PhotoDownload.as_view(),
        name='app_gallery-photo-download'),
    url(r'^photos/tag/(?P<slug>[-\w]+)/$',
        PhotosTag.as_view(),
        name='app_gallery-photos-tag'),
    url(r'^photos/tag/(?P<slug>[-\w]+)/(?P<page>\d+)/$',
        PhotosTag.as_view(),
        name='app_gallery-photos-tag-page'),
    url(r'^photo/add/tag/$',
        PhotoAddTag.as_view(),
        name='app_gallery-photo-add-tag'),
    url(r'^photo/get_video/$',
        PhotoGetVideo.as_view(),
        name='app_gallery-photo-getvideo'),
    url(r'^videos/$',
        Videos.as_view(),
        name='app_gallery-videos'),
    url(r'^videos/page/(?P<page>\d+)/$',
        Videos.as_view(),
        name='app_gallery-videos-page'),
    url(r'^(?P<folder>[-_A-Za-z0-9]+)/$',
        PhotosFolder.as_view(),
        name='app_gallery-folder'),
)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Photo.folder'
        db.add_column('rgallery_photo', 'folder',
                      self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='photo', null=True, to=orm['rgallery.Folder']),
                      keep_default=False)

        # Adding field 'Video.folder'
        db.add_column('rgallery_video', 'folder',
                      self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='video', null=True, to=orm['rgallery.Folder']),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Photo.folder'
        db.delete_column('rgallery_photo', 'folder_id')

        # Deleting field 'Video.folder'
        db.delete_column('rgallery_video', 'folder_id')


    models = {
        'rgallery.folder': {
            'Meta': {'object_name': 'Folder'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'rgallery.photo': {
            'Meta': {'object_name': 'Photo'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'folder': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'photo'", 'null': 'True', 'to': "orm['rgallery.Folder']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'rgallery.video': {
            'Meta': {'object_name': 'Video'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'folder': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'video'", 'null': 'True', 'to': "orm['rgallery.Folder']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'video': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'blank': 'True'})
        }
    }

    complete_apps = ['rgallery']


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Photo.video'
        db.add_column('rgallery_photo', 'video',
                      self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True),
                      keep_default=False)


        # Changing field 'Video.video'
        db.alter_column('rgallery_video', 'video', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True))

    def backwards(self, orm):
        # Deleting field 'Photo.video'
        db.delete_column('rgallery_photo', 'video')


        # Changing field 'Video.video'
        db.alter_column('rgallery_video', 'video', self.gf('django.db.models.fields.files.FileField')(default='', max_length=100))

    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'rgallery.folder': {
            'Meta': {'object_name': 'Folder'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'rgallery.photo': {
            'Meta': {'object_name': 'Photo'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'folder': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'photo'", 'null': 'True', 'to': "orm['rgallery.Folder']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'video': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'rgallery.video': {
            'Meta': {'object_name': 'Video'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'folder': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'video'", 'null': 'True', 'to': "orm['rgallery.Folder']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'video': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'taggit.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100'})
        },
        'taggit.taggeditem': {
            'Meta': {'object_name': 'TaggedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'taggit_taggeditem_tagged_items'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'taggit_taggeditem_items'", 'to': "orm['taggit.Tag']"})
        }
    }

    complete_apps = ['rgallery']


# -*- coding: utf-8 -*-

from django.conf.urls import patterns, include, url
from django.conf.urls.static import static
from django.conf import settings

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',

    # Admin
    url(r'^tinymce/', include('tinymce.urls')),
    url(r'^admin/filebrowser/', include('filebrowser.urls')),
    url(r'^admin/', include(admin.site.urls)),

    # App_Flatpages
    url(r'^rgallery/', include('rgallery.urls')),
)

# Media files (only for devel)
if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^media/(?P<path>.*)$', 'django.views.static.serve', {
            'document_root': settings.MEDIA_ROOT,
        }),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from optparse import make_option
from django.core.management.base import BaseCommand
from django.template import Context, Template
from django.conf import settings as conf
from sorl.thumbnail import get_thumbnail

import rgallery.models as mymodels
from utils import *


# Class MUST be named 'Command'
class Command(BaseCommand):

    # Displayed from 'manage.py help mycommand'
    help = """
    Tool that tries to download and parse photos from a source bucket,
    saving it on the database and converting the photos to fit the web:

    ./manage.py mediasync

    1.- First time it connects with the bucket and if it's Dropbox, stores the
        token information in a token_file
    2.- Next times it read the token from file to connect (only Dropbox)
    3.- Go to the shared folder and check if the image is already on database
    4.- If not, this script download the image, converting it to fit the web and
        save a record in the database.

    To run this script from a crontab task we should do something like this:

    */30 * * * * cd /path/to/rgallery-project/ ; source env/bin/activate ; \
                 cd src ; python manage.py mediasync > /dev/null

    Options:
        --storage=[dropbox|file]
        --source=/path/to/photos (only when storage=file)
        --tags=peques,tag2,tag3

    """

    # If there is a HOMETAG, set is as the default tag importing photos
    try:
        default_tag = conf.HOMETAG
    except:
        default_tag = None

    option_list = BaseCommand.option_list + (
        make_option(
            '--storage',
            dest='storage',
            default='dropbox',
            help='Backend storage where the photos are'
        ),
        make_option(
            '--source',
            dest='source',
            default='/Users/oscar/Desktop/Peques/',
            help='Where the photos are in --storage=file'
        ),
        make_option(
            '--tags',
            dest='tags',
            default=default_tag,
            help='Tag(s) to add to photos --tag=tag1,tag2,tag3'
        ),
    )

    def handle(self, *app_labels, **options):
        """
        The command itself
        """

        # Vars and folders
        storage = options['storage']
        source = options['source']
        try:
            tags = [x.strip() for x in options['tags'].split(',')]
        except:
            tags = ''
        img_duped = 0
        vid_duped = 0
        img_total = 0
        vid_total = 0
        total = 0
        thumbs = conf.RGALLLERY_THUMBS

        # print "*"*80
        # print tags
        # print len(tags)
        # print "*"*80
        # exit(0)

        # Handle connection with the backend (dinamically importing backend)
        # backend_file, backend_dropbox...
        backend = __import__(
            'rgallery.management.commands.backend_%s' % storage,
            fromlist='*')

        print "-"*80
        if options['tags']:
                print "[WWW] Photos are being imported with [%s] tags..." % \
                    options['tags']
        print "[***] Importing..."
        print ""

        # Set dirs (src, dest)
        srcdir, photodestdir, videodestdir = backend.set_dirs(source)

        # Handle connection with the backend
        client, bucket = backend.get_contents(srcdir)

        for file in bucket:
            img_duped, img_total, vid_duped, vid_total, total = mediasync(
                file,
                srcdir,
                photodestdir,
                videodestdir,
                thumbs,
                backend,
                client,
                img_duped,
                vid_duped,
                img_total,
                vid_total,
                total,
                tags)

        print ""
        print "-"*80
        print ""
        print "[***] Making missing thumbs"
        print ""
        conf.THUMBNAIL_DEBUG = True
        allphotos = mymodels.Photo.objects.all()
        for i, p in enumerate(allphotos):
            print "%04d - %s" % (i, p)
            for thumb in thumbs:
                c = Context({'image': p, 'thumb': "%sx%s" % (thumb, thumb)})
                t = Template('{% load thumbnail %}{% thumbnail image thumb crop="top" as img %}{{ img.url }}{% endthumbnail %}')
                t.render(c)
                get_thumbnail(p, "%sx%s" % (thumb, thumb))

        print ""
        print "-"*80
        print ""
        print "[***] Resume images"
        print ""
        print "[***] Duped images (ddbb): %s" % img_duped
        print "[***] Total images: %s" % img_total
        print ""
        print "[***] Resume videos"
        print ""
        print "[***] Duped videos (ddbb): %s" % vid_duped
        print "[***] Total videos: %s" % vid_total
        print ""
        print "[***] TOTAL: %s" % total
        print ""
        print "-"*80



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Photo'
        db.create_table('rgallery_photo', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100, blank=True)),
            ('origen', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('insert_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('capture_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal('rgallery', ['Photo'])

        # Adding model 'Video'
        db.create_table('rgallery_video', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100, blank=True)),
            ('video', self.gf('django.db.models.fields.files.FileField')(max_length=100, blank=True)),
            ('origen', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('insert_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('capture_date', self.gf('django.db.models.fields.DateTimeField')()),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal('rgallery', ['Video'])


    def backwards(self, orm):
        # Deleting model 'Photo'
        db.delete_table('rgallery_photo')

        # Deleting model 'Video'
        db.delete_table('rgallery_video')


    models = {
        'rgallery.photo': {
            'Meta': {'object_name': 'Photo'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'})
        },
        'rgallery.video': {
            'Meta': {'object_name': 'Video'},
            'capture_date': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'blank': 'True'}),
            'insert_date': ('django.db.models.fields.DateTimeField', [], {}),
            'origen': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'video': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'blank': 'True'})
        }
    }

    complete_apps = ['rgallery']


# -*- coding: utf-8 -*-

import os
import errno
import re
from datetime import datetime
from subprocess import Popen, PIPE

from django.template.defaultfilters import slugify
from django.template import Context, Template
from django.conf import settings as conf

import Image
import ExifTags
from ExifTags import TAGS
from sorl.thumbnail import get_thumbnail
from shutil import copyfile
from taggit.models import Tag

from rgallery.engine import expire_view_cache

import rgallery.models as mymodels


# GENERAL **********************************************************************

def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:  # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def namedup(nombre, photodestdir):
    nombre_imagen = "%s%s" % (slugify(
        str(os.path.splitext(nombre)[0]).replace('/', '').replace(' ','_').replace(':','-')),
        str(os.path.splitext(nombre)[1]))

    # Descargar original a img (si ya existe _1.jpg)
    if os.path.exists(os.path.join(photodestdir, nombre_imagen)) == True:
        nombre_imagen = "%s_1%s" % (slugify(
            str(os.path.splitext(nombre)[0]).replace('/', '').replace(' ','_').replace(':','-')),
            str(os.path.splitext(nombre)[1]))
        print "      Duped in disk, name changed to: %s" % nombre_imagen
    return nombre_imagen


# IMAGES ***********************************************************************

def img_get_exif(fn):
    """
    data = get_exif('img/2013-04-13 12.17.09.jpg')
    print data
    """

    ret = {}
    i = Image.open(fn)
    if hasattr(i, '_getexif'):
        info = i._getexif()
        try:
            for tag, value in info.items():
                decoded = TAGS.get(tag, tag)
                ret[decoded] = value
        except:
            now = datetime.now()
            ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

        try:
            str(ret['DateTimeOriginal'])
        except:
            now = datetime.now()
            ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

    return ret


def img_rotate(im2):
    try:
        # Rotar si es necesario
        for orientation in ExifTags.TAGS.keys():
            if ExifTags.TAGS[orientation] == 'Orientation':
                break
        exif = dict(im2._getexif().items())
        if exif[orientation] == 3:
            im2 = im2.rotate(180, expand=True)
        elif exif[orientation] == 6:
            im2 = im2.rotate(270, expand=True)
        elif exif[orientation] == 8:
            im2 = im2.rotate(90, expand=True)
    except:
        pass

    return im2


# VIDEOS ***********************************************************************

def video_get_exif(fn):
    # data = get_exif('img/2013-04-13 12.17.09.jpg')
    # print data

    rotate = '0'
    creation = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S")
    process = Popen([conf.FFPROBE, '-show_streams', str(fn)],
                    stdout=PIPE,
                    stderr=PIPE)
    stdout, stderr = process.communicate()

    for line in iter(stdout.splitlines()):
        if re.search(r'TAG:rotate', line):
            rotate = line.split('=')[1]
        if re.search(r'TAG:creation_time', line):
            creation = line.split('=')[1]

    return rotate, creation


def video_convert(video, file, srcdir, destdir, video_name, rotate):
    # Convertir con FFMPEG      (en principio no hace falta convertir...)
    # Hacer thumb               (...pero hace falta hacer thumb)

    # ffmpeg -i [sourcefile.avi] -acodec mp3 -ar 22050 -ab 32 -f flv -s \
    # 320×240 [destfile.flv]
    # flvtool2 -U [flvfile]
    # ffmpeg -y -i [videofile] -vframes 1 -ss 00:00:02 -an -vcodec png -f \
    # rawvideo -s 320×240 [thumbnailimage.png]
    #
    # convertvideo = "%s -i %s -acodec %s -ar 22050 -ab 32 -f flv -s %s %s" % \
    #    (conf.ZERO14_FFMPEG, src, conf.ZERO14_FFMPEG_ACODEC, \
    #    conf.ZERO14_FFMPEG_VIDEO_SIZE, flvfile)
    # MarcosBL:
    # ffmpeg -vstats_file /tmp/vstats -i "$1" -f mp4 -vcodec libx264 -r 25 -b \
    # 560000 -ab 64 -ar 44100 -threads 0 "$2" 2>/dev/null &

    try:
        src = file.path
    except:
        src = video
    dst = os.path.join(destdir, video_name)
    thumbname = "%s%s" % (slugify(str(os.path.splitext(os.path.basename(video_name))[0])),'.png')
    thumb = os.path.join(destdir, thumbname)

    rotate, data_video = video_get_exif(dst)

    transpose = ''
    size = conf.FFMPEG_THUMB_SIZE
    if rotate == '90':
        transpose = '-vf "transpose=1"'
        size = conf.FFMPEG_THUMB_SIZE_INVERSE

    # Conf
    # FFPROBE='/opt/local/bin/ffprobe'
    # FFMPEG='/opt/local/bin/ffmpeg'
    # FFMPEG_VCODEC_THUMB='png'
    # FFMPEG_THUMB_SIZE='444x250'
    # FFMPEG_THUMB_SIZE_INVERSE='250x444'

    grabimage = "%s -y -i '%s' -vframes 1 -ss 00:00:00 -an -vcodec %s -f rawvideo -s %s %s %s " % (
        conf.FFMPEG, src,
        conf.FFMPEG_VCODEC_THUMB,
        size,
        transpose,
        thumb)
    #os.system(convertvideo)
    #print "      [%s]" % grabimage
    os.system(grabimage)

    # Copy file to dest
    try:
        copyfile(src, dst)
    except:
        # If backend_form, src and dest are equal, so it fails. That's the
        # main reason of this try..except
        pass

    return thumbname


# MEDIASYNC ********************************************************************

def mediasync(file, srcdir, photodestdir, videodestdir, thumbs, backend,
              client, img_duped, vid_duped, img_total, vid_total, total, tags):

    filename = os.path.basename(backend.filepath(file))
    if backend.is_image(file):
        try:
            im = mymodels.Photo.objects.get(origen=filename)
            img_duped += 1
            print "%04d - Duped: %s" % (img_duped, filename)
        except:
            # Downloading
            print "%04d - Image %s not in database, downlaoding, thumbing and adding to database" % (total, filename)
            # If duped adds '_1' to the name
            nombre_imagen = namedup(filename, photodestdir)
            img = backend.download(client, file, nombre_imagen,
                                   srcdir, photodestdir)
            print "      Saved: %s/%s" % (photodestdir, nombre_imagen)

            # Reading EXIF Data
            data_image = img_get_exif(img)
            try:
                capture_data = datetime.strptime(
                    str(data_image['DateTimeOriginal']),
                    "%Y:%m:%d %H:%M:%S")
            except:
                capture_data = datetime.now()

            # Adding to database
            im = mymodels.Photo(image="uploads/photos/" + nombre_imagen,
                                origen=filename,
                                insert_date=datetime.now(),
                                capture_date=capture_data,
                                status=True)
            im.save()
            if len(tags) > 0:
                for tag in tags:
                    try:
                        t = Tag.objects.get(slug=tag)
                        im.tags.add(t)
                    except:
                        im.tags.add(tag)
            im.save()
            img_total += 1
            print "      Added to database (%s)" % im.origen

            # Open image to modify
            im2 = Image.open(img)  # Open
            im2 = img_rotate(im2)  # Rotate
            im2.save(os.path.join(photodestdir, nombre_imagen))

            # Creating thumbs
            for thumb in thumbs:
                print "      Thumb %sx%s" % (thumb, thumb)
                c = Context({'image': im, 'thumb': "%sx%s" % (thumb, thumb)})
                t = Template('{% load thumbnail %}{% thumbnail image thumb crop="top" as img %}{{ img.url }}{% endthumbnail %}')
                t.render(c)
                get_thumbnail(im, "%sx%s" % (thumb, thumb))

    if backend.is_video(file):
        try:
            im = mymodels.Photo.objects.get(origen=filename)
            vid_duped += 1
            print "%04d - Duped: %s" % (vid_duped, filename)
        except:
            # Downloading
            print "%04d - Video %s not in database, downloading, thumbing and adding to database" % (total, filename)
            # If duped adds '_1' to the name
            video_name = namedup(filename, videodestdir)
            video = backend.download(
                client, file, video_name, srcdir, videodestdir)
            print "      Saved: %s/%s" % (videodestdir, video_name)

            # Reading EXIF Data
            rotate, data_video = video_get_exif(video)
            capture_data = datetime.strptime(
                str(data_video), "%Y-%m-%d %H:%M:%S")

            # Converting + thumb
            thumbname = video_convert(
                video, file, srcdir, videodestdir, video_name, rotate)

            # Adding to database
            vid = mymodels.Photo(title=capture_data,
                                 image="uploads/videos/" + thumbname,
                                 video="uploads/videos/" + video_name,
                                 origen=filename,
                                 insert_date=datetime.now(),
                                 capture_date=capture_data,
                                 status=True)
            vid.save()
            vid_total += 1
            print "      Added to database (%s)" % vid.origen

            # Creating thumbs of image-video (poster)
            for thumb in thumbs:
                print "      Thumb %sx%s" % (thumb, thumb)
                get_thumbnail(vid.image, "%sx%s" % (thumb, thumb))
    total += 1
    expire_view_cache("app_gallery-gallery")
    return img_duped, img_total, vid_duped, vid_total, total



# -*- coding: utf-8 -*-
import os
import re
import sys
import time
import datetime
import gzip
import glob

from django.conf import settings as conf
from dropbox import client, rest, session

from utils import *


def set_dirs(source):
    source = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox')
    photo_target = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'photos')
    video_target = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'videos')
    if os.path.exists(photo_target) == False:
        mkdir_p(photo_target)
    if os.path.exists(video_target) == False:
        mkdir_p(video_target)
    return source, photo_target, video_target


def get_contents(srcdir):
    """
    Handle Dropbox connection
    """
    # Get your app key and secret from the Dropbox developer website
    APP_KEY = conf.DROPBOX_APP_KEY
    APP_SECRET = conf.DROPBOX_APP_SECRET

    # ACCESS_TYPE should be 'dropbox' or 'app_folder' as configured for your app
    try:
        ACCESS_TYPE = conf.DROPBOX_ACCESS_TYPE
    except:
        ACCESS_TYPE = 'dropbox'

    # Creating dirs
    if os.path.exists(srcdir) == False:
        mkdir_p(srcdir)

    # First time
    TOKENS = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox', 'dropbox_token.txt')
    if not os.path.exists(TOKENS):
        sess = session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
        request_token = sess.obtain_request_token()
        url = sess.build_authorize_url(request_token)

        # Make the user sign in and authorize this token
        print "url:", url
        print "Please visit this website and press the 'Allow' button, then hit 'Enter' here."
        raw_input()

        # This will fail if the user didn't visit the above URL
        access_token = sess.obtain_access_token(request_token)

        token_file = open(TOKENS,'w')
        token_file.write("%s|%s" % (access_token.key,access_token.secret) )
        token_file.close()
    else:
        token_file = open(TOKENS)
        token_key,token_secret = token_file.read().split('|')
        token_file.close()
        sess = session.DropboxSession(APP_KEY,APP_SECRET, ACCESS_TYPE )
        sess.set_token(token_key,token_secret)

    c = client.DropboxClient(sess)

    #if debug:
    #    print "linked account:", c.account_info()

    # DROPBOX_PATH: If the access is 'dropbox' you have to specify the folder
    # If it's 'app_folder' it's ok with /
    try:
        DROPBOX_PATH = conf.DROPBOX_ACCESS_PATH
    except:
        DROPBOX_PATH = '/'

    folder_metadata = c.metadata(DROPBOX_PATH)
    #if debug:
    #    print "*"*80
    #    print "metadata:", folder_metadata
    #    print "*"*80
    #    print "contents:", folder_metadata['contents']
    #    print "-"*80

    #if debug:
    #    for file in folder_metadata['contents']:
    #        print file

    return c, folder_metadata['contents']


def filepath(file):
    return file['path']


def is_image(cont):
    try:
        str(cont['mime_type'])
    except:
        cont['mime_type'] = ''

    if cont['mime_type'] == 'image/jpeg':
        return True
    return False


def is_video(cont):
    try:
        str(cont['mime_type'])
    except:
        cont['mime_type'] = ''

    if cont['mime_type'] == 'video/mp4' or cont['mime_type'] == 'video/3gpp':
        return True
    return False


def download(client, file, nombre_imagen, srcdir, destdir):
    f, metadata = client.get_file_and_metadata(file['path'])
    img = os.path.join(srcdir, nombre_imagen)
    out = open(img, 'wb')
    out.write(f.read())
    out.close()
    return img
